@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Perry Lorier 
  *          
  * All rights reserved.
@@ -23,7 +23,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_bpf.c 1265 2007-09-05 05:46:01Z perry $
+ * $Id: format_bpf.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -61,6 +61,8 @@ struct libtrace_format_data_t {
 	unsigned int buffersize;
 	int remaining;
 	unsigned int linktype;
+	struct bpf_stat stats;
+	int stats_valid;
 };
 
 #define FORMATIN(x) ((struct libtrace_format_data_t*)((x->format_data)))
@@ -74,6 +76,7 @@ static int bpf_init_input(libtrace_t *libtrace)
 	FORMATIN(libtrace)->fd = -1;
 	FORMATIN(libtrace)->promisc = 0;
 	FORMATIN(libtrace)->snaplen = 65536;
+	FORMATIN(libtrace)->stats_valid = 0;
 
 	return 0;
 }
@@ -192,12 +195,42 @@ static int bpf_start_input(libtrace_t *libtrace)
 		}
 	}
 
+	FORMATIN(libtrace)->stats_valid = 0;
+
 	/* TODO: we should always set a bpf filter for snapping */
 
 	/* We're done! */
 	return 0;
 }
 
+static uint64_t bpf_get_received_packets(libtrace_t *trace)
+{
+	/* If we're called with stats_valid == 0, or we're called again
+	 * then refresh the stats.  Don't refresh the stats if we're called
+	 * immediately after get_dropped_packets
+	 */
+	if (FORMATIN(trace)->stats_valid != 2) {
+		ioctl(FORMATIN(trace)->fd, BIOCGSTATS, &FORMATIN(trace)->stats);
+		FORMATIN(trace)->stats_valid = 1;
+	}
+
+	return FORMATIN(trace)->stats.bs_recv;
+}
+
+static uint64_t bpf_get_dropped_packets(libtrace_t *trace)
+{
+	/* If we're called with stats_valid == 0, or we're called again
+	 * then refresh the stats.  Don't refresh the stats if we're called
+	 * immediately after get_received_packets
+	 */
+	if (!FORMATIN(trace)->stats_valid != 1) {
+		ioctl(FORMATIN(trace)->fd, BIOCGSTATS, &FORMATIN(trace)->stats);
+		FORMATIN(trace)->stats_valid = 2;
+	}
+
+	return FORMATIN(trace)->stats.bs_drop;
+}
+
 static int bpf_pause_input(libtrace_t *libtrace)
 {
 	close(FORMATIN(libtrace)->fd);
@@ -333,7 +366,7 @@ static int bpf_get_fd(const libtrace_t *trace) {
 }
 
 static void bpf_help() {
-	printf("bpf format module: $Revision: 1265 $\n");
+	printf("bpf format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tbpf:\n");
 	printf("\n");
@@ -341,7 +374,7 @@ static void bpf_help() {
 }
 static struct libtrace_format_t bpf = {
 	"bpf",
-	"$Id: format_bpf.c 1265 2007-09-05 05:46:01Z perry $",
+	"$Id: format_bpf.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_BPF,
 	bpf_init_input,	 	/* init_input */
 	bpf_config_input,	/* config_input */
@@ -368,6 +401,10 @@ static struct libtrace_format_t bpf = {
 	bpf_get_wire_length,	/* get_wire_length */
 	bpf_get_framing_length,	/* get_framing_length */
 	NULL,			/* set_capture_length */
+	bpf_get_received_packets,/* get_received_packets */
+	NULL,			/* get_filtered_packets */
+	bpf_get_dropped_packets,/* get_dropped_packets */
+	NULL,			/* get_captured_packets */
 	bpf_get_fd,		/* get_fd */
 	trace_event_device,	/* trace_event */
 	bpf_help,		/* help */
