@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson
  *          Perry Lorier
  *          Shane Alcock
@@ -25,7 +25,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_dag24.c 1255 2007-09-04 22:18:58Z spa1 $
+ * $Id: format_dag24.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 #define _GNU_SOURCE
@@ -79,6 +79,7 @@ struct dag_format_data_t {
 	uint32_t offset;
 	uint32_t bottom;
 	uint32_t top;
+	uint64_t drops;
 };
 
 static int dag_available(libtrace_t *libtrace) {
@@ -129,6 +130,7 @@ static int dag_init_input(libtrace_t *libtrace) {
         DUCK.duck_freq = 0;
         DUCK.last_pkt = 0;
         DUCK.dummy_duck = NULL;
+	FORMAT_DATA->drops = 0;
 
         return 0;
 }
@@ -163,6 +165,7 @@ static int dag_start_input(libtrace_t *libtrace) {
 	/* Flush the memory hole */
 	while(dag_available(libtrace) != 0)
 		FORMAT_DATA->diff = 0;
+	FORMAT_DATA->drops = 0;
 	return 0;
 }
 
@@ -287,6 +290,7 @@ static int dag_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
         dag_form_packet(erfptr, packet);
         tv = trace_get_timeval(packet);
         DUCK.last_pkt = tv.tv_sec;
+	DATA(libtrace)->drops += ntohs(erfptr->lctr);
         return packet->payload ? htons(erfptr->rlen) : erf_get_framing_length(packet);
 }
 
@@ -322,8 +326,13 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
         return event;
 }
 
+static uint64_t dag24_get_dropped_packets(libtrace_t *trace)
+{
+	return DATA(trace)->drops;
+}
+
 static void dag_help(void) {
-        printf("dag format module: $Revision: 1255 $\n");
+        printf("dag format module: $Revision: 1319 $\n");
         printf("Supported input URIs:\n");
         printf("\tdag:/dev/dagn\n");
         printf("\n");
@@ -336,7 +345,7 @@ static void dag_help(void) {
 
 static struct libtrace_format_t dag = {
         "dag",
-        "$Id: format_dag24.c 1255 2007-09-04 22:18:58Z spa1 $",
+        "$Id: format_dag24.c 1319 2008-01-02 21:10:07Z perry $",
         TRACE_FORMAT_ERF,
         dag_init_input,                 /* init_input */
         dag_config_input,               /* config_input */
@@ -363,6 +372,10 @@ static struct libtrace_format_t dag = {
         erf_get_wire_length,            /* get_wire_length */
         erf_get_framing_length,         /* get_framing_length */
         erf_set_capture_length,         /* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	dag24_get_dropped_packets,	/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
         NULL,                           /* get_fd */
         trace_event_dag,                /* trace_event */
         dag_help,                       /* help */
