@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson
  *          Perry Lorier
  *	    Shane Alcock
@@ -25,7 +25,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_dag25.c 1255 2007-09-04 22:18:58Z spa1 $
+ * $Id: format_dag25.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 #define _GNU_SOURCE
@@ -78,6 +78,7 @@ struct dag_format_data_t {
 	uint8_t *bottom;
 	uint8_t *top;
 	uint32_t processed;
+	uint64_t drops;
 };
 
 static int dag_init_input(libtrace_t *libtrace) {
@@ -109,6 +110,7 @@ static int dag_init_input(libtrace_t *libtrace) {
         DUCK.last_pkt = 0;
         DUCK.dummy_duck = NULL;
 	FORMAT_DATA->stream_attached = 0;
+	FORMAT_DATA->drops = 0;
 	
         return 0;
 }
@@ -180,6 +182,7 @@ static int dag_start_input(libtrace_t *libtrace) {
 	FORMAT_DATA->top = NULL;
 	FORMAT_DATA->bottom = NULL;
 	FORMAT_DATA->processed = 0;
+	FORMAT_DATA->drops = 0;
 	
 	return 0;
 }
@@ -328,6 +331,7 @@ static int dag_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	dag_form_packet(erfptr, packet);
 	tv = trace_get_timeval(packet);
         DUCK.last_pkt = tv.tv_sec;
+	DATA(libtrace)->drops += ntohs(erfptr->lctr);
 	return packet->payload ? htons(erfptr->rlen) : 
 				erf_get_framing_length(packet);
 }
@@ -367,7 +371,7 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
 
 
 static void dag_help(void) {
-        printf("dag format module: $Revision: 1255 $\n");
+        printf("dag format module: $Revision: 1319 $\n");
         printf("Supported input URIs:\n");
         printf("\tdag:/dev/dagn\n");
         printf("\n");
@@ -380,7 +384,7 @@ static void dag_help(void) {
 
 static struct libtrace_format_t dag = {
         "dag",
-        "$Id: format_dag25.c 1255 2007-09-04 22:18:58Z spa1 $",
+        "$Id: format_dag25.c 1319 2008-01-02 21:10:07Z perry $",
         TRACE_FORMAT_ERF,
         dag_init_input,                 /* init_input */
         dag_config_input,               /* config_input */
@@ -407,6 +411,10 @@ static struct libtrace_format_t dag = {
         erf_get_wire_length,            /* get_wire_length */
         erf_get_framing_length,         /* get_framing_length */
         erf_set_capture_length,         /* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	dag25_get_dropped_packets,	/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
         NULL,                           /* get_fd */
         trace_event_dag,                /* trace_event */
         dag_help,                       /* help */
