@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_duck.c 1195 2007-04-21 03:59:56Z perry $
+ * $Id: format_duck.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -271,7 +271,7 @@ static void duck_help(void) {
 }
 static struct libtrace_format_t duck = {
         "duck",
-        "$Id: format_duck.c 1195 2007-04-21 03:59:56Z perry $",
+        "$Id: format_duck.c 1319 2008-01-02 21:10:07Z perry $",
         TRACE_FORMAT_DUCK,
         duck_init_input,	        /* init_input */
         NULL,                           /* config_input */
@@ -298,6 +298,10 @@ static struct libtrace_format_t duck = {
         duck_get_wire_length,  		/* get_wire_length */
         duck_get_framing_length, 	/* get_framing_length */
         NULL,         			/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
         NULL,                           /* get_fd */
         NULL,              		/* trace_event */
         duck_help,                     	/* help */
