@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_erf.c 1246 2007-07-20 03:26:04Z spa1 $
+ * $Id: format_erf.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 #define _GNU_SOURCE
@@ -84,6 +84,7 @@ struct erf_format_data_t {
 	struct {
 		int real_time;
 	} options;
+	uint64_t drops;
 };
 
 struct erf_format_data_out_t {
@@ -147,6 +148,7 @@ static int erf_init_input(libtrace_t *libtrace)
 	
 	INPUT.file = 0;
 	IN_OPTIONS.real_time = 0;
+	DATA(libtrace)->drops = 0;
 	
 	return 0; /* success */
 }
@@ -176,6 +178,8 @@ static int erf_start_input(libtrace_t *libtrace)
 	if (!INPUT.file)
 		return -1;
 
+	DATA(libtrace)->drops = 0;
+
 	return 0; /* success */
 }
 
@@ -617,8 +621,13 @@ static struct libtrace_eventobj_t erf_event(struct libtrace_t *libtrace, struct
 	
 }
 
+static uint64_t erf_get_dropped_packets(libtrace_t *trace)
+{
+	return DATA(trace)->drops;
+}
+
 static void erf_help(void) {
-	printf("erf format module: $Revision: 1246 $\n");
+	printf("erf format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\terf:/path/to/file\t(uncompressed)\n");
 	printf("\terf:/path/to/file.gz\t(gzip-compressed)\n");
@@ -643,7 +652,7 @@ static void erf_help(void) {
 
 static struct libtrace_format_t erfformat = {
 	"erf",
-	"$Id: format_erf.c 1246 2007-07-20 03:26:04Z spa1 $",
+	"$Id: format_erf.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_ERF,
 	erf_init_input,			/* init_input */	
 	erf_config_input,		/* config_input */
@@ -670,6 +679,10 @@ static struct libtrace_format_t erfformat = {
 	erf_get_wire_length,		/* get_wire_length */
 	erf_get_framing_length,		/* get_framing_length */
 	erf_set_capture_length,		/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	erf_get_dropped_packets,	/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	erf_event,			/* trace_event */
 	erf_help,			/* help */
