@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_legacy.c 1243 2007-07-03 01:34:06Z spa1 $
+ * $Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 #define _GNU_SOURCE
@@ -269,11 +269,19 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 }
 		
 
-static libtrace_linktype_t legacypos_get_link_type(const libtrace_packet_t *packet UNUSED) {
+static libtrace_linktype_t legacypos_get_link_type(
+		const libtrace_packet_t *packet) {
+	/* Is this a cisco hdlc frame? */
+	if ((((uint8_t*)packet->payload)[0] == 0x0F /* Unicast */
+		|| ((uint8_t*)packet->payload)[0] == 0x8F /* Multicast */)
+		&& ((uint8_t*)packet->payload)[1] == 0x00 /* control == 0x00 */
+	   )
+		return TRACE_TYPE_HDLC_POS;
 	return TRACE_TYPE_PPP;
 }
 
-static libtrace_linktype_t legacyatm_get_link_type(const libtrace_packet_t *packet UNUSED) {
+static libtrace_linktype_t legacyatm_get_link_type(
+		const libtrace_packet_t *packet UNUSED) {
 	return TRACE_TYPE_ATM;
 }
 
@@ -366,7 +374,7 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 }	
 
 static void legacypos_help(void) {
-	printf("legacypos format module: $Revision: 1243 $\n");
+	printf("legacypos format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacypos:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacypos:/path/to/file.gz\t(gzip-compressed)\n");
@@ -377,7 +385,7 @@ static void legacypos_help(void) {
 }
 
 static void legacyatm_help(void) {
-	printf("legacyatm format module: $Revision: 1243 $\n");
+	printf("legacyatm format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyatm:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyatm:/path/to/file.gz\t(gzip-compressed)\n");
@@ -388,7 +396,7 @@ static void legacyatm_help(void) {
 }
 
 static void legacyeth_help(void) {
-	printf("legacyeth format module: $Revision: 1243 $\n");
+	printf("legacyeth format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyeth:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyeth:/path/to/file.gz\t(gzip-compressed)\n");
@@ -399,7 +407,7 @@ static void legacyeth_help(void) {
 }
 
 static void legacynzix_help(void) {
-	printf("legacynzix format module: $Revision: 1243 $\n");
+	printf("legacynzix format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacynzix:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacynzix:/path/to/file.gz\t(gzip-compressed)\n");
@@ -411,7 +419,7 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1243 2007-07-03 01:34:06Z spa1 $",
+	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_LEGACY_ATM,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -438,6 +446,10 @@ static struct libtrace_format_t legacyatm = {
 	legacyatm_get_wire_length,	/* get_wire_length */
 	legacyatm_get_framing_length,	/* get_framing_length */
 	NULL,				/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyatm_help,			/* help */
@@ -446,7 +458,7 @@ static struct libtrace_format_t legacyatm = {
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1243 2007-07-03 01:34:06Z spa1 $",
+	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_LEGACY_ETH,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -473,6 +485,10 @@ static struct libtrace_format_t legacyeth = {
 	legacyeth_get_wire_length,	/* get_wire_length */
 	legacyeth_get_framing_length,	/* get_framing_length */
 	NULL,				/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacyeth_help,			/* help */
@@ -481,7 +497,7 @@ static struct libtrace_format_t legacyeth = {
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1243 2007-07-03 01:34:06Z spa1 $",
+	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_LEGACY_POS,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -508,6 +524,10 @@ static struct libtrace_format_t legacypos = {
 	legacypos_get_wire_length,	/* get_wire_length */
 	legacypos_get_framing_length,	/* get_framing_length */
 	NULL,				/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacypos_help,			/* help */
@@ -516,7 +536,7 @@ static struct libtrace_format_t legacypos = {
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1243 2007-07-03 01:34:06Z spa1 $",
+	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_LEGACY_NZIX,
 	legacynzix_init_input,		/* init_input */	
 	NULL,				/* config_input */
@@ -543,6 +563,10 @@ static struct libtrace_format_t legacynzix = {
 	legacynzix_get_wire_length,	/* get_wire_length */
 	legacynzix_get_framing_length,	/* get_framing_length */
 	NULL,				/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	legacynzix_help,		/* help */
