@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_linux.c 1258 2007-09-04 23:28:00Z perry $
+ * $Id: format_linux.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -50,11 +50,20 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 
+#include <assert.h>
+
+struct tpacket_stats {
+	unsigned int tp_packets;
+	unsigned int tp_drops;
+};
 
 struct libtrace_format_data_t {
 	int fd;
 	int snaplen;
 	int promisc;
+	libtrace_filter_t *filter;
+	struct tpacket_stats stats;
+	int stats_valid;
 };
 
 struct libtrace_linuxnative_header {
@@ -78,6 +87,8 @@ static int linuxnative_init_input(libtrace_t *libtrace)
 	FORMAT(libtrace->format_data)->fd = -1;
 	FORMAT(libtrace->format_data)->promisc = -1;
 	FORMAT(libtrace->format_data)->snaplen = 65536;
+	FORMAT(libtrace->format_data)->filter = NULL;
+	FORMAT(libtrace->format_data)->stats_valid = 0;
 
 	return 0;
 }
@@ -96,6 +107,7 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 	struct sockaddr_ll addr;
 	int one = 1;
 	memset(&addr,0,sizeof(addr));
+	libtrace_filter_t *filter = FORMAT(libtrace->format_data)->filter;
 	FORMAT(libtrace->format_data)->fd = 
 				socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 	if (FORMAT(libtrace->format_data)->fd==-1) {
@@ -157,6 +169,34 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 		perror("setsockopt(SO_TIMESTAMP)");
 	}
 
+	/* Push BPF filter into the kernel. At this stage we can safely assume
+	 * that the filterstring has been compiled, or the filter was supplied
+	 * pre-compiled.
+	 */
+	if (filter != NULL) {
+		assert(filter->flag == 1);
+		if (setsockopt(FORMAT(libtrace->format_data)->fd,
+					SOL_SOCKET,
+					SO_ATTACH_FILTER,
+					&filter->filter,
+					sizeof(filter->filter)) == -1) {
+			perror("setsockopt(SO_ATTACH_FILTER)");
+		} else { 
+			/* The socket accepted the filter, so we need to
+			 * consume any buffered packets that were received
+			 * between opening the socket and applying the filter.
+			 */
+			void *buf = malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
+			while(recv(FORMAT(libtrace->format_data)->fd,
+					buf,
+					(size_t) LIBTRACE_PACKET_BUFSIZE,
+					MSG_DONTWAIT) != -1) { }
+			free(buf);
+		}
+	}
+
+	FORMAT(libtrace->format_data)->stats_valid=0;
+					
 	return 0;
 }
 
@@ -168,6 +208,7 @@ static int linuxnative_start_output(libtrace_out_t *libtrace)
 		free(libtrace->format_data);
 		return -1;
 	}
+	FORMAT(libtrace->format_data)->stats_valid=0;
 
 	return 0;
 }
@@ -182,7 +223,10 @@ static int linuxnative_pause_input(libtrace_t *libtrace)
 
 static int linuxnative_fin_input(libtrace_t *libtrace) 
 {
+	if (FORMAT(libtrace->format_data)->filter != NULL)
+		free(FORMAT(libtrace->format_data)->filter);
 	free(libtrace->format_data);
+	
 	return 0;
 }
 
@@ -194,6 +238,68 @@ static int linuxnative_fin_output(libtrace_out_t *libtrace)
 	return 0;
 }
 
+static int linuxnative_configure_bpf(libtrace_t *libtrace, 
+		libtrace_filter_t *filter) {
+#ifdef HAVE_LIBPCAP 
+	struct ifreq ifr;
+	unsigned int arphrd;
+	libtrace_dlt_t dlt;
+	libtrace_filter_t *f;
+	int sock;
+	pcap_t *pcap;
+
+	/* Take a copy of the filter object as it was passed in */
+	f = (libtrace_filter_t *) malloc(sizeof(libtrace_filter_t));
+	memcpy(f, filter, sizeof(libtrace_filter_t));
+	
+	/* If we are passed a filter with "flag" set to zero, then we must
+	 * compile the filterstring before continuing. This involves
+	 * determining the linktype, passing the filterstring to libpcap to
+	 * compile, and saving the result for trace_start() to push into the
+	 * kernel.
+	 * If flag is set to one, then the filter was probably generated using
+	 * trace_create_filter_from_bytecode() and so we don't need to do
+	 * anything (we've just copied it above).
+	 */
+	if (f->flag == 0) {
+		sock = socket(PF_INET, SOCK_STREAM, 0);
+		memset(&ifr, 0, sizeof(struct ifreq));
+		strncpy(ifr.ifr_name, libtrace->uridata, IF_NAMESIZE);
+		if (ioctl(sock, SIOCGIFHWADDR, &ifr) != 0) {
+			perror("Can't get HWADDR for interface");
+			return -1;
+		}
+		close(sock);
+
+		arphrd = ifr.ifr_hwaddr.sa_family;
+		dlt = libtrace_to_pcap_dlt(arphrd_type_to_libtrace(arphrd));
+
+		pcap = pcap_open_dead(dlt, 
+				FORMAT(libtrace->format_data)->snaplen);
+
+		if (pcap_compile(pcap, &f->filter, f->filterstring, 0, 0) == -1) {
+			perror("PCAP failed to compile the filterstring");
+			return -1;
+		}
+
+		pcap_close(pcap);
+		
+		/* Set the "flag" to indicate that the filterstring has been
+		 * compiled
+		 */
+		f->flag = 1;
+	}
+	
+	if (FORMAT(libtrace->format_data)->filter != NULL)
+		free(FORMAT(libtrace->format_data)->filter);
+	
+	FORMAT(libtrace->format_data)->filter = f;
+	
+	return 0;
+#else
+	return -1
+#endif
+}
 static int linuxnative_config_input(libtrace_t *libtrace,
 		trace_option_t option,
 		void *data)
@@ -206,11 +312,8 @@ static int linuxnative_config_input(libtrace_t *libtrace,
 			FORMAT(libtrace->format_data)->promisc=*(int*)data;
 			return 0;
 		case TRACE_OPTION_FILTER:
-			/* We don't support bpf filters in any special way
-			 * so return an error and let libtrace deal with
-			 * emulating it
-			 */
-			break;
+		 	return linuxnative_configure_bpf(libtrace, 
+					(libtrace_filter_t *) data);
 		case TRACE_OPTION_META_FREQ:
 			/* No meta-data for this format */
 			break;
@@ -349,7 +452,7 @@ static libtrace_direction_t linuxnative_get_direction(const struct libtrace_pack
 }
 
 static libtrace_direction_t linuxnative_set_direction(
-		const libtrace_packet_t *packet,
+		libtrace_packet_t *packet,
 		libtrace_direction_t direction) {
 
 	switch (direction) {
@@ -389,8 +492,51 @@ static int linuxnative_get_fd(const libtrace_t *trace) {
 	return FORMAT(trace->format_data)->fd;
 }
 
+/* Linux doesn't keep track how many packets were seen before filtering
+ * so we can't tell how many packets were filtered.  Bugger.  So annoying.
+ *
+ * Since we tell libtrace that we do support filtering, if we don't declare
+ * this here as failing, libtrace will happily report for us that it didn't
+ * filter any packets, so don't lie -- return that we don't know.
+ */
+static uint64_t linuxnative_get_filtered_packets(libtrace_t *trace) {
+	return UINT64_MAX;
+}
+
+/* Number of packets that past filtering */
+static uint64_t linuxnative_get_captured_packets(libtrace_t *trace) {
+	if (FORMAT(trace->format_data)->stats_valid != 2) {
+		socklen_t len = sizeof(FORMAT(trace->format_data)->stats);
+		getsockopt(FORMAT(trace->format_data)->fd, 
+				SOL_PACKET,
+				PACKET_STATISTICS,
+				&FORMAT(trace->format_data)->stats,
+				&len);
+		FORMAT(trace->format_data)->stats_valid = 1;
+	}
+
+	return FORMAT(trace->format_data)->stats.tp_packets;
+}
+
+/* Number of packets that got past filtering and were then dropped because
+ * of lack of space
+ */
+static uint64_t linuxnative_get_dropped_packets(libtrace_t *trace) {
+	if (FORMAT(trace->format_data)->stats_valid != 1) {
+		socklen_t len = sizeof(FORMAT(trace->format_data)->stats);
+		getsockopt(FORMAT(trace->format_data)->fd, 
+				SOL_PACKET,
+				PACKET_STATISTICS,
+				&FORMAT(trace->format_data)->stats,
+				&len);
+		FORMAT(trace->format_data)->stats_valid = 2;
+	}
+
+	return FORMAT(trace->format_data)->stats.tp_drops;
+}
+
 static void linuxnative_help(void) {
-	printf("linuxnative format module: $Revision: 1258 $\n");
+	printf("linuxnative format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tint:\n");
 	printf("\n");
@@ -401,7 +547,7 @@ static void linuxnative_help(void) {
 }
 static struct libtrace_format_t linuxnative = {
 	"int",
-	"$Id: format_linux.c 1258 2007-09-04 23:28:00Z perry $",
+	"$Id: format_linux.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_LINUX_NATIVE,
 	linuxnative_init_input,	 	/* init_input */
 	linuxnative_config_input,	/* config_input */
@@ -428,6 +574,10 @@ static struct libtrace_format_t linuxnative = {
 	linuxnative_get_wire_length,	/* get_wire_length */
 	linuxnative_get_framing_length,	/* get_framing_length */
 	NULL,				/* set_capture_length */
+	NULL,				/* get_received_packets */
+	linuxnative_get_filtered_packets,/* get_filtered_packets */
+	linuxnative_get_dropped_packets,/* get_dropped_packets */
+	linuxnative_get_captured_packets,/* get_captured_packets */
 	linuxnative_get_fd,		/* get_fd */
 	trace_event_device,		/* trace_event */
 	linuxnative_help,		/* help */
