@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $
+ * $Id: format_pcap.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -91,6 +91,7 @@ static int pcap_init_input(libtrace_t *libtrace) {
 static int pcap_start_input(libtrace_t *libtrace) {
 	char errbuf[PCAP_ERRBUF_SIZE];
 
+
 	/* if the file is already open */
 	if (INPUT.pcap)
 		return 0; /* success */
@@ -103,8 +104,13 @@ static int pcap_start_input(libtrace_t *libtrace) {
 		return -1;
 	}
 	if (DATA(libtrace)->filter) {
-		pcap_compile(INPUT.pcap, &DATA(libtrace)->filter->filter,
-				DATA(libtrace)->filter->filterstring, 1, 0);
+		if (DATA(libtrace)->filter->flag == 0) {
+			pcap_compile(INPUT.pcap, 
+					&DATA(libtrace)->filter->filter,
+					DATA(libtrace)->filter->filterstring, 
+					1, 0);
+			DATA(libtrace)->filter->flag = 1;
+		}
 		if (pcap_setfilter(INPUT.pcap,&DATA(libtrace)->filter->filter) 
 				== -1) {
 			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,"%s",
@@ -312,17 +318,24 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 		libtrace_packet_t *packet) 
 {
 	struct pcap_pkthdr pcap_pkt_hdr;
+	void *link;
+	libtrace_linktype_t linktype;
+	uint32_t remaining;
+
+	link = trace_get_packet_buffer(packet,&linktype,&remaining);
 
 	/* If this packet cannot be converted to a pcap linktype then
 	 * pop off the top header until it can be converted
 	 */
-	while (libtrace_to_pcap_linktype(trace_get_link_type(packet))==~0U) {
+	while (libtrace_to_pcap_linktype(linktype)==~0U) {
 		if (!demote_packet(packet)) {
 			trace_set_err_out(libtrace, 
 				TRACE_ERR_NO_CONVERSION,
 				"pcap does not support this format");
 			return -1;
 		}
+
+		link = trace_get_packet_buffer(packet,&linktype,&remaining);
 	}
 
 
@@ -345,7 +358,7 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 	}
 
 	/* Corrupt packet, or other "non data" packet, so skip it */
-	if (trace_get_link(packet) == NULL) {
+	if (link == NULL) {
 		/* Return "success", but nothing written */
 		return 0;
 	}
@@ -362,7 +375,7 @@ static int pcap_write_packet(libtrace_out_t *libtrace,
 		struct timeval ts = trace_get_timeval(packet);
 		pcap_pkt_hdr.ts.tv_sec = ts.tv_sec;
 		pcap_pkt_hdr.ts.tv_usec = ts.tv_usec;
-		pcap_pkt_hdr.caplen = trace_get_capture_length(packet);
+		pcap_pkt_hdr.caplen = remaining;
 		/* trace_get_wire_length includes FCS, while pcap doesn't */
 		if (trace_get_link_type(packet)==TRACE_TYPE_ETH)
 			if (trace_get_wire_length(packet) >= 4) { 
@@ -441,7 +454,8 @@ static libtrace_direction_t pcap_get_direction(const libtrace_packet_t *packet)
 		case TRACE_TYPE_LINUX_SLL:
 		{
 			libtrace_sll_header_t *sll;
-			sll = trace_get_link(packet);
+			sll = trace_get_packet_buffer(packet, NULL, NULL);
+			/* TODO: should check remaining>=sizeof(*sll) */
 			if (!sll) {
 				trace_set_err(packet->trace,
 					TRACE_ERR_BAD_PACKET,
@@ -470,7 +484,8 @@ static libtrace_direction_t pcap_get_direction(const libtrace_packet_t *packet)
 		case TRACE_TYPE_PFLOG:
 		{
 			libtrace_pflog_header_t *pflog;
-			pflog = trace_get_link(packet);
+			pflog = trace_get_packet_buffer(packet, NULL, NULL);
+			/* TODO: should check remaining >= sizeof(*pflog) */
 			if (!pflog) {
 				trace_set_err(packet->trace,
 						TRACE_ERR_BAD_PACKET,
@@ -517,13 +532,15 @@ static int pcap_get_wire_length(const libtrace_packet_t *packet) {
 	if (packet->type==pcap_linktype_to_rt(TRACE_DLT_EN10MB))
 		return pcapptr->len+4; /* Include the missing FCS */
 	else if (packet->type==pcap_linktype_to_rt(TRACE_DLT_IEEE802_11_RADIO)) {
+		libtrace_linktype_t linktype;
+		void *link = trace_get_packet_buffer(packet,&linktype,NULL);
 		/* If the packet is Radiotap and the flags field indicates
 		 * that the FCS is not included in the 802.11 frame, then
 		 * we need to add 4 to the wire-length to account for it.
 		 */
 		uint8_t flags;
-		trace_get_wireless_flags(trace_get_link(packet), 
-				trace_get_link_type(packet), &flags);
+		trace_get_wireless_flags(link, 
+				linktype, &flags);
 		if ((flags & TRACE_RADIOTAP_F_FCS) == 0)
 			return pcapptr->len + 4;
 	}
@@ -552,8 +569,22 @@ static int pcap_get_fd(const libtrace_t *trace) {
 	return pcap_fileno(DATA(trace)->input.pcap);
 }
 
+static uint64_t pcap_get_dropped_packets(libtrace_t *trace)
+{
+	struct pcap_stat stats;
+	if (pcap_stats(DATA(trace)->input.pcap,&stats)==-1) {
+		char *errmsg = pcap_geterr(DATA(trace)->input.pcap);
+		trace_set_err(trace,TRACE_ERR_UNSUPPORTED,
+				"Failed to retreive stats: %s\n",
+				errmsg ? errmsg : "Unknown pcap error");
+		return ~0;
+	}
+
+	return stats.ps_drop;
+}
+
 static void pcap_help(void) {
-	printf("pcap format module: $Revision: 1247 $\n");
+	printf("pcap format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcap:/path/to/file\n");
 	printf("\n");
@@ -565,7 +596,7 @@ static void pcap_help(void) {
 }
 
 static void pcapint_help(void) {
-	printf("pcapint format module: $Revision: 1247 $\n");
+	printf("pcapint format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapint:interface\n");
 	printf("\n");
@@ -579,7 +610,7 @@ static void pcapint_help(void) {
 
 static struct libtrace_format_t pcap = {
 	"pcap",
-	"$Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $",
+	"$Id: format_pcap.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_PCAP,
 	pcap_init_input,		/* init_input */
 	pcap_config_input,		/* config_input */
@@ -606,6 +637,10 @@ static struct libtrace_format_t pcap = {
 	pcap_get_wire_length,		/* get_wire_length */
 	pcap_get_framing_length,	/* get_framing_length */
 	pcap_set_capture_length,	/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	pcap_help,			/* help */
@@ -614,7 +649,7 @@ static struct libtrace_format_t pcap = {
 
 static struct libtrace_format_t pcapint = {
 	"pcapint",
-	"$Id: format_pcap.c 1247 2007-08-01 04:58:15Z spa1 $",
+	"$Id: format_pcap.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_PCAP,
 	pcapint_init_input,		/* init_input */
 	pcapint_config_input,		/* config_input */
@@ -641,6 +676,10 @@ static struct libtrace_format_t pcapint = {
 	pcap_get_wire_length,		/* get_wire_length */
 	pcap_get_framing_length,	/* get_framing_length */
 	pcap_set_capture_length,	/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	pcap_get_dropped_packets,	/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	pcap_get_fd,			/* get_fd */
 	trace_event_device,		/* trace_event */
 	pcapint_help,			/* help */
