@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcapfile.c 1246 2007-07-20 03:26:04Z spa1 $
+ * $Id: format_pcapfile.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -280,17 +280,30 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 	struct timeval tv = trace_get_timeval(packet);
 	int numbytes;
 	int ret;
+	void *ptr;
+	uint32_t remaining;
+	libtrace_linktype_t linktype;
+
+	ptr = trace_get_packet_buffer(packet,&linktype,&remaining);
+	
+	/* Silently discard RT metadata packets and packets with an
+	 * unknown linktype. */
+	if (linktype == TRACE_TYPE_METADATA || linktype == -1) {
+		return 0;
+	}
 
 	/* If this packet cannot be converted to a pcap linktype then
 	 * pop off the top header until it can be converted
 	 */
-	while (libtrace_to_pcap_linktype(trace_get_link_type(packet))==~0U) {
+	while (libtrace_to_pcap_linktype(linktype)==~0U) {
 		if (!demote_packet(packet)) {
 			trace_set_err_out(out, 
 				TRACE_ERR_NO_CONVERSION,
 				"pcap does not support this format");
 			return -1;
 		}
+
+		ptr = trace_get_packet_buffer(packet,&linktype,&remaining);
 	}
 
 
@@ -313,19 +326,19 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 		pcaphdr.sigfigs = 0;
 		pcaphdr.snaplen = 65536;
 		pcaphdr.network = 
-			libtrace_to_pcap_linktype(trace_get_link_type(packet));
+			libtrace_to_pcap_linktype(linktype);
 
-		libtrace_io_write(DATAOUT(out)->file, &pcaphdr, sizeof(pcaphdr));
+		libtrace_io_write(DATAOUT(out)->file, 
+				&pcaphdr, sizeof(pcaphdr));
 	}
 
 	hdr.ts_sec = tv.tv_sec;
 	hdr.ts_usec = tv.tv_usec;
 	hdr.caplen = trace_get_capture_length(packet);
 	/* PCAP doesn't include the FCS, we do */
-	if (trace_get_link_type(packet)==TRACE_TYPE_ETH)
+	if (linktype==TRACE_TYPE_ETH)
 		if (trace_get_wire_length(packet) >= 4) {
-			hdr.wirelen =
-					trace_get_wire_length(packet)-4;
+			hdr.wirelen = trace_get_wire_length(packet)-4;
 		}
 		else {
 			hdr.wirelen = 0;
@@ -341,10 +354,10 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 		return -1;
 
 	ret=libtrace_io_write(DATAOUT(out)->file,
-			trace_get_link(packet),
-			trace_get_capture_length(packet));
+			ptr,
+			remaining);
 
-	if (ret!=(int)trace_get_capture_length(packet))
+	if (ret!=(int)remaining)
 		return -1;
 
 	return numbytes+ret;
@@ -370,7 +383,12 @@ static libtrace_direction_t pcapfile_get_direction(const libtrace_packet_t *pack
 		case TRACE_TYPE_LINUX_SLL:
 		{
 			libtrace_sll_header_t *sll;
-			sll = (libtrace_sll_header_t*)trace_get_link(packet);
+			libtrace_linktype_t linktype;
+
+			sll = (libtrace_sll_header_t*)trace_get_packet_buffer(
+					packet,
+					&linktype,
+					NULL);
 			if (!sll) {
 				trace_set_err(packet->trace,
 					TRACE_ERR_BAD_PACKET,
@@ -399,7 +417,10 @@ static libtrace_direction_t pcapfile_get_direction(const libtrace_packet_t *pack
 		case TRACE_TYPE_PFLOG:
 		{
 			libtrace_pflog_header_t *pflog;
-			pflog=(libtrace_pflog_header_t*)trace_get_link(packet);
+			libtrace_linktype_t linktype;
+
+			pflog=(libtrace_pflog_header_t*)trace_get_packet_buffer(
+					packet,&linktype,NULL);
 			if (!pflog) {
 				trace_set_err(packet->trace,
 						TRACE_ERR_BAD_PACKET,
@@ -454,8 +475,10 @@ static int pcapfile_get_wire_length(const libtrace_packet_t *packet) {
 		 * we need to add 4 to the wire-length to account for it.
 		 */
 		uint8_t flags;
-		trace_get_wireless_flags(trace_get_link(packet),
-				trace_get_link_type(packet), &flags);
+		void *link;
+		libtrace_linktype_t linktype;
+		link = trace_get_packet_buffer(packet, &linktype, NULL);
+		trace_get_wireless_flags(link, linktype, &flags);
 		if ((flags & TRACE_RADIOTAP_F_FCS) == 0)
 			return swapl(packet->trace,pcapptr->wirelen)+4;
 	}
@@ -495,7 +518,7 @@ static struct libtrace_eventobj_t pcapfile_event(libtrace_t *libtrace, libtrace_
 }
 
 static void pcapfile_help(void) {
-	printf("pcapfile format module: $Revision: 1246 $\n");
+	printf("pcapfile format module: $Revision: 1319 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tpcapfile:/path/to/file\n");
 	printf("\tpcapfile:/path/to/file.gz\n");
@@ -506,7 +529,7 @@ static void pcapfile_help(void) {
 
 static struct libtrace_format_t pcapfile = {
 	"pcapfile",
-	"$Id: format_pcapfile.c 1246 2007-07-20 03:26:04Z spa1 $",
+	"$Id: format_pcapfile.c 1319 2008-01-02 21:10:07Z perry $",
 	TRACE_FORMAT_PCAPFILE,
 	pcapfile_init_input,		/* init_input */
 	pcapfile_config_input,		/* config_input */
@@ -533,6 +556,10 @@ static struct libtrace_format_t pcapfile = {
 	pcapfile_get_wire_length,	/* get_wire_length */
 	pcapfile_get_framing_length,	/* get_framing_length */
 	pcapfile_set_capture_length,	/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	pcapfile_help,			/* help */
