@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson
  *          Perry Lorier
  *	    Shane Alcock
@@ -25,7 +25,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_rt.c 1263 2007-09-05 04:34:10Z perry $
+ * $Id: format_rt.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -306,7 +306,6 @@ static int rt_read(libtrace_t *libtrace, void **buffer, size_t len, int block)
 		memcpy(RT_INFO->pkt_buffer, RT_INFO->buf_current, 
 				RT_INFO->buf_filled);
 		RT_INFO->buf_current = RT_INFO->pkt_buffer;
-		
 #ifndef MSG_NOSIGNAL
 #  define MSG_NOSIGNAL 0
 #endif
@@ -362,8 +361,11 @@ static int rt_read(libtrace_t *libtrace, void **buffer, size_t len, int block)
 
 static int rt_set_format(libtrace_t *libtrace, libtrace_packet_t *packet) 
 {
-	
-	if (packet->type >= TRACE_RT_DATA_DLT) {
+
+	/* Try to minimize the number of corrupt packets that slip through
+	 * while making it easy to identify new pcap DLTs */
+	if (packet->type > TRACE_RT_DATA_DLT && 
+			packet->type < TRACE_RT_DATA_DLT_END) {
 		if (!RT_INFO->dummy_pcap) {
 			RT_INFO->dummy_pcap = trace_create_dead("pcap:-");
 		}
@@ -398,7 +400,7 @@ static int rt_set_format(libtrace_t *libtrace, libtrace_packet_t *packet)
 			trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, "Legacy packet cannot be sent over rt");
 			return -1;
 		default:
-			printf("Unrecognised format: %d\n", packet->type);
+			printf("Unrecognised format: %u\n", packet->type);
 			trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, "Unrecognised packet format");
 			return -1;
 	}
@@ -488,7 +490,6 @@ static int rt_read_packet_versatile(libtrace_t *libtrace,
 	 * rt header - I know it's hax and maybe I'll fix it later on */
 	if (RT_INFO->rt_hdr.type == TRACE_RT_LAST) {
 		void_hdr = (void *)pkt_hdr;
-		
 		/* FIXME: Better error handling required */
 		if (rt_read(libtrace, &void_hdr, 
 				sizeof(rt_header_t),blocking) !=
@@ -505,6 +506,7 @@ static int rt_read_packet_versatile(libtrace_t *libtrace,
 	}
 	packet->type = RT_INFO->rt_hdr.type;
 
+	
 	if (packet->type >= TRACE_RT_DATA_SIMPLE) {
 		if (rt_read(libtrace, 
 					&packet->buffer,
@@ -578,7 +580,7 @@ static int rt_read_packet_versatile(libtrace_t *libtrace,
 	}
 	/* Return the number of bytes read from the stream */
 	RT_INFO->rt_hdr.type = TRACE_RT_LAST;
-	return RT_INFO->rt_hdr.length;
+	return RT_INFO->rt_hdr.length + sizeof(rt_header_t);
 }
 
 static int rt_read_packet(libtrace_t *libtrace,
@@ -676,6 +678,7 @@ static libtrace_eventobj_t trace_event_rt(libtrace_t *trace,
 	else {
 		event.type = TRACE_EVENT_PACKET;
 	}
+
 	
 	return event;
 }
@@ -695,7 +698,7 @@ static void rt_help(void) {
 
 static struct libtrace_format_t rt = {
         "rt",
-        "$Id: format_rt.c 1263 2007-09-05 04:34:10Z perry $",
+        "$Id: format_rt.c 1319 2008-01-02 21:10:07Z perry $",
         TRACE_FORMAT_RT,
         rt_init_input,            	/* init_input */
         NULL,                           /* config_input */
@@ -722,6 +725,10 @@ static struct libtrace_format_t rt = {
         rt_get_wire_length,            		/* get_wire_length */
         rt_get_framing_length, 		/* get_framing_length */
         NULL,         			/* set_capture_length */
+	NULL,				/* get_received_packets */
+	NULL,				/* get_filtered_packets */
+	NULL,				/* get_dropped_packets */
+	NULL,				/* get_captured_packets */
         rt_get_fd,                	/* get_fd */
         trace_event_rt,             /* trace_event */
         rt_help,			/* help */
