@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -141,11 +141,13 @@ extern "C" {
  * parameters, used to hint gcc's optimisations
  */
 #if __GNUC__ >= 3 
+#  define DEPRECATED __attribute__((deprecated))
 #  define SIMPLE_FUNCTION __attribute__((pure))
 #  define UNUSED __attribute__((unused))
 #  define PACKED __attribute__((packed))
 #  define PRINTF(formatpos,argpos) __attribute__((format(printf,formatpos,argpos)))
 #else
+#  define DEPRECATED
 #  define SIMPLE_FUNCTION
 #  define UNUSED
 #  define PACKED 
@@ -199,7 +201,9 @@ enum {
 	/** Option known, but unsupported by this format */
 	TRACE_ERR_OPTION_UNAVAIL= -6,
 	/** This feature is unsupported */
-	TRACE_ERR_UNSUPPORTED	= -7
+	TRACE_ERR_UNSUPPORTED	= -7,
+	/** Illegal use of the API */
+	TRACE_ERR_BAD_STATE	= -8
 };
 
 /** Enumeration of DLT supported by libtrace 
@@ -216,6 +220,7 @@ typedef enum {
 	TRACE_DLT_RAW = 12,
 #endif
 	TRACE_DLT_LINKTYPE_RAW = 101,
+	TRACE_DLT_C_HDLC = 104,
 	TRACE_DLT_IEEE802_11 = 105,
 	TRACE_DLT_LINUX_SLL = 113,
 	TRACE_DLT_PFLOG = 117,
@@ -313,6 +318,7 @@ typedef enum {
 	TRACE_RT_DLT_LINUX_SLL		=TRACE_RT_DATA_DLT+TRACE_DLT_LINUX_SLL,
 	TRACE_RT_DLT_PFLOG		=TRACE_RT_DATA_DLT+TRACE_DLT_PFLOG,
 	TRACE_RT_DLT_ATM_RFC1483 	=TRACE_RT_DATA_DLT+TRACE_DLT_ATM_RFC1483,
+	TRACE_RT_DATA_DLT_END		= 2999,
 	TRACE_RT_LAST			= (2<<31)
 } libtrace_rt_types_t;
 
@@ -393,19 +399,7 @@ typedef struct libtrace_ip
     uint8_t  ip_tos;			/**< Type of Service */
     uint16_t ip_len;			/**< Total Length */
     int16_t  ip_id;			/**< Identification */
-#if BYTE_ORDER == LITTLE_ENDIAN
-    LT_BITFIELD16 ip_off:12;		/**< Fragment Offset */
-    LT_BITFIELD16 ip_mf:1;		/**< More Fragments Flag */
-    LT_BITFIELD16 ip_df:1;		/**< Dont Fragment Flag */
-    LT_BITFIELD16 ip_rf:1;		/**< Reserved Fragment Flag */
-#elif BYTE_ORDER == BIG_ENDIAN
-    LT_BITFIELD16 ip_rf:1;		/**< Fragment Offset */
-    LT_BITFIELD16 ip_df:1;		/**< More Fragments Flag */
-    LT_BITFIELD16 ip_mf:1;		/**< Dont Fragment Flag */
-    LT_BITFIELD16 ip_off:12;		/**< Reserved Fragment Flag */
-#else
-#   error "Adjust your <bits/endian.h> defines"
-#endif
+    uint16_t ip_off;			/**< IP Fragment offset (and flags) */
     uint8_t  ip_ttl;			/**< Time to Live */
     uint8_t  ip_p;			/**< Protocol */
     uint16_t ip_sum;			/**< Checksum */
@@ -839,6 +833,40 @@ DLLEXPORT bool trace_is_err_output(libtrace_out_t *trace);
 DLLEXPORT void trace_perror_output(libtrace_out_t *trace, const char *msg,...)
 	PRINTF(2,3);
 
+/** Return the number of captured packets 
+ * Includes the number of packets counted as early as possible, before
+ * filtering, and includes dropped packets.
+ *
+ * @param trace		Trace to examine
+ * @returns number of packets seen at the capture point before filtering.
+ *
+ * If this is not known, this will return UINT64_MAX
+ */
+uint64_t trace_get_received_packets(libtrace_t *trace);
+
+/** Return the number of filtered packets
+ * Returns the number of packets that were captured, but discarded for not
+ * matching a trace filter.  This includes packets 
+ *
+ * @param trace		Trace file to examine
+ * @returns the number of packets that were successfully captured, but filtered
+ *
+ * If this is not known, this will return UINT64_MAX
+ */
+uint64_t trace_get_filtered_packets(libtrace_t *trace);
+
+/** Return the number of packets that have been dropped for lack of packets
+ * @param trace		Trace file to examine
+ * @returns The number of packets captured, but dropped due to buffer overruns
+ */
+uint64_t trace_get_dropped_packets(libtrace_t *trace);
+
+/** Return the number of packets that have been returned to library user
+ * @param trace		Trace file to examine
+ * @returns The number of packets returned to the user of the library.
+ */
+uint64_t trace_get_accepted_packets(libtrace_t *trace);
+
 
 /*@}*/
 
@@ -940,21 +968,40 @@ DLLEXPORT int trace_write_packet(libtrace_out_t *trace, libtrace_packet_t *packe
  * @{
  */
 
+
+/** Gets a pointer to the first byte of the packet as it was captured and
+ * returns its corresponding linktype and capture length.
+ *
+ * Use this function instead of the deprecated trace_get_link().
+ *
+ * @param packet the packet pointer
+ * @param[out] linktype the linktype of the returned pointer
+ * @param[out] remaining the capture length (the number of captured bytes from
+ * the returned pointer)
+ * @return a pointer to the first byte of the packet
+ */
+DLLEXPORT void *trace_get_packet_buffer(const libtrace_packet_t *packet,
+                libtrace_linktype_t *linktype, uint32_t *remaining);
+
 /** get a pointer to the link layer
  * @param packet  	the packet opaque pointer
  *
  * @return a pointer to the link layer, or NULL if there is no link layer
  *
+ * @deprecated This function is deprecated: Use trace_get_packet_buffer() or
+ * one of the trace_get_layer*() functions instead.
  * @note you should call trace_get_link_type to find out what type of link
  * layer this is
  */
-DLLEXPORT SIMPLE_FUNCTION
+DLLEXPORT SIMPLE_FUNCTION DEPRECATED
 void *trace_get_link(const libtrace_packet_t *packet);
 
 /** get a pointer to the IPv4 header (if any)
  * @param packet  	the packet opaque pointer
  *
  * @return a pointer to the IPv4 header, or NULL if there is no IPv4 header
+ *
+ * You should consider using \ref trace_get_layer3 instead of this function.
  */
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet);
@@ -963,10 +1010,113 @@ libtrace_ip_t *trace_get_ip(libtrace_packet_t *packet);
  * @param packet  	the packet opaque pointer
  *
  * @return a pointer to the IPv6 header, or NULL if there is no IPv6 header
+ *
+ * You should consider using \ref trace_get_layer3 instead of this function.
  */
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_ip6_t *trace_get_ip6(libtrace_packet_t *packet);
 
+/** Return a pointer to the first metadata header in a packet, if present.
+ *
+ * This function takes a pointer to a libtrace packet and if any metadata
+ * headers exist, returns a pointer to the first one, along with its
+ * corresponding linktype. 
+ *
+ * If no metadata headers exist in the packet, NULL is returned.
+ *
+ * A metadata header is a header that was prepended by the capturing device,
+ * such as a Linux SLL header, or a Radiotap wireless monitoring header.
+ * Subsequent metadata headers may be accessed with the
+ * trace_get_payload_from_meta(...) function. 
+ *
+ * @param packet the libtrace packet
+ * @param[out] linktype the linktype of the returned metadata header
+ * @param[out] remaining the number of bytes captured after the returned
+ * pointer.
+ * @return a pointer to the first metadata header, or NULL if there are no
+ * metadata headers present.
+ *
+ * remaining may be NULL, however linktype must be provided.
+ */
+DLLEXPORT void *trace_get_packet_meta(const libtrace_packet_t *packet,
+                libtrace_linktype_t *linktype,
+                uint32_t *remaining);
+
+/** Returns the payload of a metadata header.
+ * 
+ * This function takes a pointer to the start of a metadata header (either
+ * obtained via trace_get_packet_meta(...) or by a previous call to
+ * trace_get_payload_from_meta(...)) along with its corresponding linktype and
+ * returns the payload, i.e. the next header. It will also update the linktype
+ * parameter to indicate the type of payload.
+ *  
+ * If the linktype indicates that the header passed in is not a metadata
+ * header, the function returns NULL to indicate this. The linktype remains
+ * unchanged in this case.
+ *
+ * This function allows the user to iterate through metadata headers which are
+ * sometimes present before the actual packet as it was received on the wire.
+ * Examples of metadata headers include the Linux SLL header and the Radiotap
+ * wireless monitoring header.
+ *
+ * @param[in] meta a pointer to a header
+ * @param[in,out] linktype the linktype of meta (updated to indicate the
+ * linktype of the returned header if applicable).
+ * @param[in,out] remaining the number of bytes after the meta pointer.
+ * @return a pointer to the payload of the metadata header. If meta is not a
+ * pointer to a metadata header, NULL is returned and linktype remains
+ * unchanged.
+ *
+ * All parameters are mandatory. NULL will be returned if any are NULL.
+ */
+DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
+                libtrace_linktype_t *linktype,
+                uint32_t *remaining);
+
+
+/** Get a pointer to the layer 2 header. Generally this is the first byte of the
+ * packet as it was seen on the wire.
+ * 
+ * This function takes a libtrace packet and skips over any metadata headers if
+ * present (such as Linux SLL or Radiotap) and returns a pointer to the first
+ * byte of the packet that was actually received by the network interface.
+ *
+ * @param packet the libtrace packet
+ * @param[out] linktype the linktype of the returned layer 2 header
+ * @param[out] remaining the number of bytes left in the packet after the
+ * returned pointer.
+ * @return a pointer to the first byte of the packet as it was seen on the
+ * wire.
+ *
+ * remaining may be NULL, otherwise it will be filled in by the function.
+ */
+DLLEXPORT void *trace_get_layer2(const libtrace_packet_t *packet,
+                libtrace_linktype_t *linktype,
+                uint32_t *remaining);
+
+/** Gets a pointer to the next header given a pointer to a layer2 header
+ *
+ * @param l2            	The pointer to the current layer2 header
+ * @param linktype		The type of the layer2 header
+ * @param[out] ethertype 	An optional output variable of the ethernet type
+ * @param[in,out] remaining 	Optionally updated with the length remaining
+ *
+ * @return a pointer to the transport layer header, or NULL if header isn't
+ * present.
+ *
+ * type may be NULL if not needed.
+ *
+ * Remaining may be NULL.  If Remaining is not NULL it must point to the number
+ * of bytes captured of the layer2 header and beyond.  It will be decremented
+ * by the number of bytes skipped to find the payload.
+ *
+ */
+DLLEXPORT void *trace_get_payload_from_layer2(void *l2,
+		libtrace_linktype_t linktype,
+		uint16_t *ethertype,
+		uint32_t *remaining);
+
+
 /** Get a pointer to the layer 3 header.
  * @param packet  		The packet opaque pointer
  * @param[out] ethertype	The ethertype of the layer 3 header
@@ -974,35 +1124,36 @@ libtrace_ip6_t *trace_get_ip6(libtrace_packet_t *packet);
  * 				has been removed.
  *
  * @return a pointer to the layer 3 header.
- * remaining may be NULL, otherwise it will be filled in by the remaining size
- * of the captured packet.
+ * remaining may be NULL, otherwise it will be set to the number of captured
+ * bytes after the pointer returned.
  */
-DLLEXPORT SIMPLE_FUNCTION
-void *trace_get_layer3(libtrace_packet_t *packet,
+DLLEXPORT 
+void *trace_get_layer3(const libtrace_packet_t *packet,
 		uint16_t *ethertype, uint32_t *remaining);
 
 /** Gets a pointer to the transport layer header (if any)
  * @param packet        a pointer to a libtrace_packet structure
  * @param[out] proto	transport layer protocol
  *
- * @return a pointer to the transport layer header, or NULL if there is no header
+ * @return a pointer to the transport layer header, or NULL if there is no
+ * header
  *
  * @note proto may be NULL if proto is unneeded.
  */
-DLLEXPORT void *trace_get_transport(libtrace_packet_t *packet, uint8_t *proto, 
-		uint32_t *remaining);
+DLLEXPORT void *trace_get_transport(const libtrace_packet_t *packet, 
+		uint8_t *proto, uint32_t *remaining);
 
 /** Gets a pointer to the payload given a pointer to the IP header
  * @param ip            The IP Header
  * @param[out] proto	An output variable of the IP protocol
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
- * @return a pointer to the transport layer header, or NULL if header isn't present.
+ * @return a pointer to the transport layer header, or NULL if header isn't
+ * present.
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the IP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the IP header (and any IP options)
- * have been removed.
+ * of bytes captured of the IP header and beyond.  It will be decremented by
+ * the length of the IPv4 header (including any options).
  *
  * proto may be NULL if not needed.
  *
@@ -1016,12 +1167,12 @@ DLLEXPORT void *trace_get_payload_from_ip(libtrace_ip_t *ip, uint8_t *proto,
  * @param[out] proto    An output variable of the protocol of the next header
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
- * @return a pointer to the transport layer header, or NULL if the IPv6 header isn't complete.
+ * @return a pointer to the transport layer header, or NULL if the IPv6 header
+ * isn't complete.
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the IP header and beyond.  It will be updated after this
- * function to the number of bytes remaining after the IP header (and any IP options)
- * have been removed.
+ * of bytes captured of the IP header and beyond.  It will be decremented by
+ * this function by the length of the IPV6 header.
  *
  * proto may be NULL if not needed.
  *
@@ -1050,7 +1201,8 @@ DLLEXPORT void *trace_get_payload_from_link(void *linkptr,
 		uint16_t *type, uint32_t *remaining);
 
 /** Skips over any 802.1q headers, if present.
- * @param ethernet      A pointer to the payload following an ethernet header -usually the result of calling trace_get_payload_from_link
+ * @param ethernet      A pointer to the payload following an ethernet header
+ * -usually the result of calling trace_get_payload_from_link
  * @param[in,out] type  The ethernet type, replaced with the vlan ether type
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
@@ -1058,9 +1210,8 @@ DLLEXPORT void *trace_get_payload_from_link(void *linkptr,
  * Otherwise, returns the ethernet payload that was passed in
  *
  * Remaining may be NULL. If Remaining is not NULL it must point to the number
- * of bytes captured past (but not including) the link layer. It will be
- * updated after this function to the number of bytes remaining after the
- * vlan header. If it is unchanged then no vlan header exists.
+ * of bytes captured past (but not including) the link layer. This function
+ * will decrement it by the length of the 802.1q headers if present.
  *
  * Type must point to the value of the ethernet type. Libtrace will assert
  * fail if type is NULL.
@@ -1076,9 +1227,8 @@ DLLEXPORT void *trace_get_vlan_payload_from_ethernet_payload(
  * @return a pointer to the tcp payload, or NULL if the payload isn't present.
  *
  * Remaining may be NULL.  If remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after
- * this function to the number of bytes remaining after the TCP header (and any
- * TCP options) have been removed.
+ * of bytes captured of the TCP header and beyond.  It will be decremented by
+ * this function by the length of the TCP header (including any options).
  *
  */
 DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp, 
@@ -1091,7 +1241,7 @@ DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp,
  * @return a pointer to the udp payload, or NULL if the payload isn't present.
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the UDP header and beyond.  It will be updated after
+ * of bytes captured of the UDP header and beyond.  It will be decremented by
  * this function to the number of bytes remaining after the UDP header.
  *
  */
@@ -1104,8 +1254,8 @@ DLLEXPORT void *trace_get_payload_from_udp(libtrace_udp_t *udp, uint32_t *remain
  * @return a pointer to the icmp payload, or NULL if the payload isn't present.
  *
  * Remaining may be NULL.  If remaining is not NULL it must point to the number
- * of bytes captured of the ICMP header and beyond.  It will be updated after
- * this function to the number of bytes remaining after the ICMP header.
+ * of bytes captured of the ICMP header and beyond.  It will be decremented
+ * by the number of bytes in the ICMP header.
  * 
  */
 DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp, 
@@ -1115,6 +1265,8 @@ DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp,
  * @param packet  	the packet opaque pointer
  *
  * @return a pointer to the TCP header, or NULL if there is not a TCP packet
+ *
+ * @note you should probably use trace_get_transport()
  */
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_tcp_t *trace_get_tcp(libtrace_packet_t *packet);
@@ -1126,9 +1278,8 @@ libtrace_tcp_t *trace_get_tcp(libtrace_packet_t *packet);
  * @return a pointer to the TCP header, or NULL if this is not a TCP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be updated after
- * this function to the number of bytes remaining after the TCP header (and any
- * TCP options) have been removed.
+ * of bytes captured of the TCP header and beyond.  It will be decremented by
+ * the number of bytes in the TCP header (including any TCP options).
  *
  * @note The last parameter has changed from libtrace2
  */
@@ -1150,9 +1301,8 @@ libtrace_udp_t *trace_get_udp(libtrace_packet_t *packet);
  * @return a pointer to the UDP header, or NULL if this is not an UDP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the UDP header and beyond.  It will be updated after
- * this function to the number of bytes remaining after the UDP header have
- * been removed.
+ * of bytes captured of the UDP header and beyond.  This function will
+ * decremented it by the length of the UDP header.
  *
  * @note Beware the change from libtrace2 from skipped to remaining
  */
@@ -1174,8 +1324,8 @@ libtrace_icmp_t *trace_get_icmp(libtrace_packet_t *packet);
  * @return a pointer to the ICMP header, or NULL if this is not an ICMP packet
  *
  * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the ICMP header and beyond.  It will be updated after
- * this function to the number of bytes remaining after the ICMP header.
+ * of bytes captured of the ICMP header and beyond.  It will be decremented by
+ * the length of the ICMP head in bytes.
  *
  * @note Beware the change from libtrace2 from skipped to remaining
  */
@@ -1315,7 +1465,7 @@ DLLEXPORT int trace_seek_erf_timestamp(libtrace_t *trace, uint64_t ts);
  * a packet can have
  * @{
  */
-/** Get the size of the packet in the trace
+/** Get the size of the packet in the trace (in bytes)
  * @param packet  	the packet opaque pointer
  * @return the size of the packet in the trace
  * @note Due to this being a header capture, or anonymisation, this may not
@@ -1330,9 +1480,9 @@ DLLEXPORT int trace_seek_erf_timestamp(libtrace_t *trace, uint64_t ts);
  * (pcap/erf/etc) header.
  */
 DLLEXPORT SIMPLE_FUNCTION
-size_t trace_get_capture_length(libtrace_packet_t *packet);
+size_t trace_get_capture_length(const libtrace_packet_t *packet);
 
-/** Get the size of the packet as it was seen on the wire.
+/** Get the size of the packet as it was seen on the wire (in bytes).
  * @param packet  	the packet opaque pointer
  * @return the size of the packet as it was on the wire.
  * @note Due to the trace being a header capture, or anonymisation this may
@@ -1347,7 +1497,7 @@ size_t trace_get_capture_length(libtrace_packet_t *packet);
 DLLEXPORT SIMPLE_FUNCTION
 size_t trace_get_wire_length(const libtrace_packet_t *packet);
 
-/** Get the length of the capture framing headers.
+/** Get the length of the capture framing headers (in bytes).
  * @param packet  	the packet opaque pointer
  * @return the size of the packet as it was on the wire.
  * @note this length corresponds to the difference between the size of a 
@@ -1358,7 +1508,7 @@ size_t trace_get_framing_length(const libtrace_packet_t *packet);
 
 /** Truncate ("snap") the packet at the suggested length
  * @param packet	the packet opaque pointer
- * @param size		the new length of the packet
+ * @param size		the new length of the packet (in bytes)
  * @return the new capture length of the packet, or the original capture
  * length of the packet if unchanged
  */
@@ -1407,7 +1557,17 @@ libtrace_direction_t trace_get_direction(const libtrace_packet_t *packet);
 DLLEXPORT SIMPLE_FUNCTION
 libtrace_filter_t *trace_create_filter(const char *filterstring);
 
-/** apply a BPF filter
+/** Setup a BPF filter based on pre-compiled byte-code.
+ * @param bf_insns	A pointer to the start of the byte-code
+ * @param bf_len	The number of BPF instructions	
+ * @returns		an opaque pointer to a libtrace_filter_t object
+ * @note		The supplied byte-code is not checked for correctness.
+ * @author		Scott Raynel
+ */
+DLLEXPORT libtrace_filter_t *
+trace_create_filter_from_bytecode(void *bf_insns, unsigned int bf_len);
+
+/** Apply a BPF filter to a packet
  * @param filter 	the filter opaque pointer
  * @param packet	the packet opaque pointer
  * @return >0 if the filter matches, 0 if it doesn't, -1 on error.
@@ -1420,7 +1580,7 @@ libtrace_filter_t *trace_create_filter(const char *filterstring);
 DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 		const libtrace_packet_t *packet);
 
-/** destory of BPF filter
+/** Destroy a BPF filter
  * @param filter 	the filter opaque pointer
  * Deallocate all the resources associated with a BPF filter
  */
