@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: libtrace_int.h 1266 2007-09-05 05:46:41Z perry $
+ * $Id: libtrace_int.h 1319 2008-01-02 21:10:07Z perry $
  *
  */
 /** @file */
@@ -118,9 +118,9 @@ int snprintf(char *str, size_t size, const char *format, ...);
 
 struct libtrace_event_status_t {
 	libtrace_packet_t *packet;
-	int psize;
 	double tdelta;
 	double trace_last_ts;
+	int psize;
 };
 
 /** The information about traces that are open 
@@ -128,17 +128,24 @@ struct libtrace_event_status_t {
  */
 struct libtrace_t {
 	struct libtrace_format_t *format; /**< format driver pointer */
-	void *format_data; /**<format data pointer */
-	bool started;			/**< if this trace has started */
-	libtrace_err_t err;		/**< error information */
 	struct libtrace_event_status_t event;	/**< the next event */
-	char *uridata;			/**< the uri of this trace */
+	void *format_data; 		/**<format data pointer */
 	struct libtrace_filter_t *filter; /**< used by libtrace if the module
 					    * doesn't support filters natively
 					    */
-	size_t snaplen;		/**< used by libtrace if the module
+	size_t snaplen;			/**< used by libtrace if the module
 					  * doesn't support snapping natively
 					  */
+	uint64_t accepted_packets;	/**< Number of packets returned to the
+					  * user 
+					  */
+	uint64_t filtered_packets;	/**< Number of packets filtered by
+					  * libtrace
+					  */
+	char *uridata;			/**< the uri of this trace */
+
+	libtrace_err_t err;		/**< error information */
+	bool started;			/**< if this trace has started */
 };
 
 /** Information about output traces
@@ -147,9 +154,9 @@ struct libtrace_t {
 struct libtrace_out_t {
         struct libtrace_format_t *format;	/**< format driver */
 	void *format_data; 		/**< format data */
-	bool started;			/**< trace started */
-	libtrace_err_t err;		/**< Associated error */
 	char *uridata;			/**< URI associated with this trace */
+	libtrace_err_t err;		/**< Associated error */
+	bool started;			/**< trace started */
 };
 
 void trace_set_err(libtrace_t *trace, int errcode,const char *msg,...) 
@@ -190,7 +197,7 @@ typedef struct libtrace_pflog_header_t {
 	uint32_t   subrulenr;
 	uint8_t	   dir;
 	uint8_t	   pad[3];
-} libtrace_pflog_header_t;
+} PACKED libtrace_pflog_header_t;
 
 
 
@@ -339,6 +346,17 @@ struct libtrace_format_t {
 	 * @note This callback may be NULL if not supported.
 	 */
 	size_t (*set_capture_length)(struct libtrace_packet_t *packet,size_t size);
+	/** Report the number of packets ever seen as early as possible
+	 */
+	uint64_t (*get_received_packets)(libtrace_t *trace);
+	/** Report the number of filtered packets.
+	 */
+	uint64_t (*get_filtered_packets)(libtrace_t *trace);
+	/** Return the number of dropped packets */
+	uint64_t (*get_dropped_packets)(libtrace_t *trace);
+	/** Report number of captured packets
+	 */
+	uint64_t (*get_captured_packets)(libtrace_t *trace);
 	/** return the filedescriptor associated with this interface.
 	 * @note This callback may be NULL if not supported.
 	 * This function is only needed if you use trace_event_interface
@@ -372,7 +390,7 @@ unsigned int libtrace_to_arphrd_type(libtrace_linktype_t);
 void promote_packet(libtrace_packet_t *packet);
 bool demote_packet(libtrace_packet_t *packet);
 
-void *trace_get_payload_from_linux_sll(void *, uint16_t *, uint32_t *);
+void *trace_get_payload_from_linux_sll(const void *, uint16_t *, uint32_t *);
 void *trace_get_payload_from_pos(void *, uint16_t *, uint32_t *);
 DLLEXPORT void *trace_get_payload_from_atm(void *, uint8_t *, uint32_t *);
 
@@ -429,8 +447,8 @@ uint16_t byteswap16(uint16_t num);
  */
 struct libtrace_filter_t {
 	struct bpf_program filter;
-	int flag;
 	char * filterstring;
+	int flag;
 };
 #else
 struct libtrace_filter_t {};
