@@ -1,7 +1,7 @@
 /*
  * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
+ * Copyright (c) 2007,2008 The University of Waikato, Hamilton, New Zealand.
  * Authors: Daniel Lawson 
  *          Perry Lorier 
  *          
@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1263 2007-09-05 04:34:10Z perry $
+ * $Id: trace.c 1319 2008-01-02 21:10:07Z perry $
  *
  */
 
@@ -129,7 +129,7 @@ static char *xstrndup(const char *src,size_t n)
 }
 
 void register_format(struct libtrace_format_t *f) {
-	assert(f->next==NULL);
+	assert(f->next==NULL); /* Can't register a format twice */
 	f->next=formats_list;
 	formats_list=f;
 	/* Now, verify things 
@@ -230,7 +230,7 @@ static void trace_init(void)
 #ifdef HAVE_LIBPCAP
 		pcap_constructor();
 #endif
-#ifdef HAVE_BIOCSETIF
+#if HAVE_DECL_BIOCSETIF
 		bpf_constructor();
 #endif
 		pcapfile_constructor();
@@ -421,6 +421,8 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
 
 	libtrace->err.err_num = TRACE_ERR_NOERROR;
 	strcpy(libtrace->err.problem,"Error message set\n");
+        libtrace->format = NULL;
+	libtrace->uridata = NULL;
 	
         /* parse the URI to determine what sort of event we are dealing with */
 
@@ -430,7 +432,6 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
 		return libtrace;
 	}
 	
-        libtrace->format = NULL;
 	for(tmp=formats_list;tmp;tmp=tmp->next) {
                 if (strlen(scan) == strlen(tmp->name) &&
                                 !strncasecmp(scan,
@@ -440,6 +441,8 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
                                 break;
                                 }
         }
+	free(scan);
+
         if (libtrace->format == NULL) {
 		trace_set_err_out(libtrace,TRACE_ERR_BAD_FORMAT,
 				"Unknown output format (%s)",scan);
@@ -447,7 +450,6 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
         }
         libtrace->uridata = strdup(uridata);
 
-
         /* libtrace->format now contains the type of uri
          * libtrace->uridata contains the appropriate data for this
 	 */
@@ -460,7 +462,7 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
 			case 0: /* success */
 				break;
 			default:
-				assert(!"init_output() should return -1 for failure, or 0 for success");
+				assert(!"Internal error: init_output() should return -1 for failure, or 0 for success");
 		}
 	} else {
 		trace_set_err_out(libtrace,TRACE_ERR_UNSUPPORTED,
@@ -469,7 +471,6 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
         }
 
 
-	free(scan);
 	libtrace->started=false;
 	return libtrace;
 }
@@ -484,7 +485,8 @@ DLLEXPORT libtrace_out_t *trace_create_output(const char *uri) {
 DLLEXPORT int trace_start(libtrace_t *libtrace)
 {
 	assert(libtrace);
-	assert(!trace_is_err(libtrace) && "Please use trace_is_err to check for errors after calling trace_create!");
+	if (trace_is_err(libtrace))
+		return -1;
 	if (libtrace->format->start_input) {
 		int ret=libtrace->format->start_input(libtrace);
 		if (ret < 0) {
@@ -513,7 +515,10 @@ DLLEXPORT int trace_start_output(libtrace_out_t *libtrace)
 DLLEXPORT int trace_pause(libtrace_t *libtrace)
 {
 	assert(libtrace);
-	assert(libtrace->started && "BUG: Called trace_pause without calling trace_start first");
+	if (!libtrace->started) {
+		trace_set_err(libtrace,TRACE_ERR_BAD_STATE, "You must call trace_start() before calling trace_pause()");
+		return -1;
+	}
 	if (libtrace->format->pause_input)
 		libtrace->format->pause_input(libtrace);
 	libtrace->started=false;
@@ -526,8 +531,10 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 {
 	int ret;
 	libtrace_err_t err;
-	
-	assert(!trace_is_err(libtrace) && "Please use trace_is_err to check for errors after calling trace_create!");
+
+	if (trace_is_err(libtrace)) {
+		return -1;
+	}
 	
 	if (libtrace->format->config_input) {
 		ret=libtrace->format->config_input(libtrace,option,value);
@@ -625,9 +632,10 @@ DLLEXPORT void trace_destroy_dead(libtrace_t *libtrace) {
 DLLEXPORT void trace_destroy_output(libtrace_out_t *libtrace) 
 {
 	assert(libtrace);
-	if (libtrace->format)
+	if (libtrace->format && libtrace->format->fin_output)
 		libtrace->format->fin_output(libtrace);
-	free(libtrace->uridata);
+	if (libtrace->uridata)
+		free(libtrace->uridata);
 	free(libtrace);
 }
 
@@ -644,10 +652,16 @@ DLLEXPORT libtrace_packet_t *trace_create_packet(void)
 DLLEXPORT libtrace_packet_t *trace_copy_packet(const libtrace_packet_t *packet) {
 	libtrace_packet_t *dest = 
 		(libtrace_packet_t *)malloc(sizeof(libtrace_packet_t));
+	if (!dest) {
+		printf("out of memory constructing packet\n");
+		abort();
+	}
 	dest->trace=packet->trace;
-	dest->buffer=malloc(
-			trace_get_framing_length(packet)
-			+trace_get_capture_length(packet));
+	dest->buffer=malloc(65536);
+	if (!dest->buffer) {
+		printf("out of memory allocating buffer memory\n");
+		abort();
+	}
 	dest->header=dest->buffer;
 	dest->payload=(void*)
 		((char*)dest->buffer+trace_get_framing_length(packet));
@@ -689,10 +703,17 @@ DLLEXPORT void trace_destroy_packet(libtrace_packet_t *packet) {
 DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 
 	assert(libtrace && "You called trace_read_packet() with a NULL libtrace parameter!\n");
-	assert(libtrace->started && "BUG: You must call libtrace_start() before trace_read_packet()\n");
+	if (trace_is_err(libtrace))
+		return -1;
+	if (!libtrace->started) {
+		trace_set_err(libtrace,TRACE_ERR_BAD_STATE,"You must call libtrace_start() before trace_read_packet()\n");
+		return -1;
+	}
+	if (!(packet->buf_control==TRACE_CTRL_PACKET || packet->buf_control==TRACE_CTRL_EXTERNAL)) {
+		trace_set_err(libtrace,TRACE_ERR_BAD_STATE,"Packet passed to trace_read_packet() is invalid\n");
+		return -1;
+	}
 	assert(packet);
-	assert((packet->buf_control==TRACE_CTRL_PACKET || packet->buf_control==TRACE_CTRL_EXTERNAL)&&
-		"BUG: You must allocate a packet using packet_create()");
       
 	/* Store the trace we are reading from into the packet opaque 
 	 * structure */
@@ -722,6 +743,7 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 				 * packet
 				 */
 				if (!trace_apply_filter(libtrace->filter,packet)){
+					++libtrace->filtered_packets;
 					continue;
 				}
 			}
@@ -730,6 +752,7 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 				trace_set_capture_length(packet,
 						libtrace->snaplen);
 			}
+			++libtrace->accepted_packets;
 			return ret;
 		} while(1);
 	}
@@ -749,7 +772,11 @@ DLLEXPORT int trace_write_packet(libtrace_out_t *libtrace, libtrace_packet_t *pa
 	assert(libtrace);
 	assert(packet);	
 	/* Verify the packet is valid */
-	assert(libtrace->started);
+	if (!libtrace->started) {
+		trace_set_err_out(libtrace,TRACE_ERR_BAD_STATE,
+			"Trace is not started before trace_write_packet");
+		return -1;
+	}
 
 	if (libtrace->format->write_packet) {
 		return libtrace->format->write_packet(libtrace, packet);
@@ -759,6 +786,14 @@ DLLEXPORT int trace_write_packet(libtrace_out_t *libtrace, libtrace_packet_t *pa
 	return -1;
 }
 
+DLLEXPORT void *trace_get_packet_buffer(const libtrace_packet_t *packet,
+		libtrace_linktype_t *linktype, uint32_t *remaining) {
+	assert(packet != NULL);
+	if (linktype) *linktype = trace_get_link_type(packet);
+	if (remaining) *remaining = trace_get_capture_length(packet);
+	return (void *) packet->payload;
+}
+
 DLLEXPORT void *trace_get_link(const libtrace_packet_t *packet) {
 	return (void *)packet->payload;
 }
@@ -860,13 +895,14 @@ DLLEXPORT double trace_get_seconds(const libtrace_packet_t *packet) {
 	return seconds;
 }
 
-DLLEXPORT size_t trace_get_capture_length(libtrace_packet_t *packet) 
+DLLEXPORT size_t trace_get_capture_length(const libtrace_packet_t *packet) 
 {
 	/* Cache the capture length */
 	if (packet->capture_length == -1) {
 		if (!packet->trace->format->get_capture_length)
 			return ~0U;
-		packet->capture_length = 
+		/* Cast away constness because this is "just" a cache */
+		((libtrace_packet_t*)packet)->capture_length = 
 			packet->trace->format->get_capture_length(packet);
 	}
 
@@ -942,25 +978,63 @@ DLLEXPORT libtrace_eventobj_t trace_event(libtrace_t *trace,
 	assert(trace);
 	assert(packet);
 
+	/* Clear the packet cache */
+	packet->capture_length = -1;
+	packet->l3_header = NULL;
+	packet->l3_ethertype = 0;
+	
 	/* Store the trace we are reading from into the packet opaque
 	 * structure */
 	packet->trace = trace;
 
 	if (packet->trace->format->trace_event) {
-		return packet->trace->format->trace_event(trace,packet);
-	} else {
-		return event;
+		event=packet->trace->format->trace_event(trace,packet);
+		if (event.type == TRACE_EVENT_PACKET) {
+			++trace->accepted_packets;
+		}
 	}
+	return event;
 
 }
 
+/** Setup a BPF filter based on pre-compiled byte-code.
+ * @param bf_insns	A pointer to the start of the byte-code
+ * @param bf_len	The number of BPF instructions
+ * @returns		an opaque pointer to a libtrace_filter_t object
+ * @note		The supplied byte-code is not checked for correctness.
+ * @author		Scott Raynel
+ */
+DLLEXPORT libtrace_filter_t *
+trace_create_filter_from_bytecode(void *bf_insns, unsigned int bf_len)
+{
+#ifndef HAVE_BPF_FILTER
+	fprintf(stderr, "This version of libtrace does not have BPF support\n");
+	return NULL;
+#else
+	struct libtrace_filter_t *filter = (struct libtrace_filter_t *)
+		malloc(sizeof(struct libtrace_filter_t));
+	filter->filter.bf_insns = (struct bpf_insn *)
+		malloc(sizeof(struct bpf_insn) * bf_len);
+	
+	memcpy(filter->filter.bf_insns, bf_insns,
+			bf_len * sizeof(struct bpf_insn));
+	
+	filter->filter.bf_len = bf_len;
+	filter->filterstring = NULL;
+	/* "flag" indicates that the filter member is valid */
+	filter->flag = 1; 
+	
+	return filter;
+#endif
+}
+
 /* setup a BPF filter
  * @param filterstring a char * containing the bpf filter string
  * @returns opaque pointer pointer to a libtrace_filter_t object
  * @author Daniel Lawson
  */
 DLLEXPORT libtrace_filter_t *trace_create_filter(const char *filterstring) {
-#ifdef HAVE_BPF
+#ifdef HAVE_BPF_FILTER
 	libtrace_filter_t *filter = (libtrace_filter_t*)
 				malloc(sizeof(libtrace_filter_t));
 	filter->filterstring = strdup(filterstring);
@@ -974,7 +1048,7 @@ DLLEXPORT libtrace_filter_t *trace_create_filter(const char *filterstring) {
 
 DLLEXPORT void trace_destroy_filter(libtrace_filter_t *filter)
 {
-#ifdef HAVE_BPF
+#ifdef HAVE_BPF_FILTER
 	free(filter->filterstring);
 	if (filter->flag)
 		pcap_freecode(&filter->filter);
@@ -991,12 +1065,13 @@ DLLEXPORT void trace_destroy_filter(libtrace_filter_t *filter)
  */
 int trace_bpf_compile(libtrace_filter_t *filter,
 		const libtrace_packet_t *packet	) {
-#ifdef HAVE_BPF
+#ifdef HAVE_BPF_FILTER
 	void *linkptr = 0;
+	libtrace_linktype_t linktype;
 	assert(filter);
 
 	/* If this isn't a real packet, then fail */
-	linkptr = trace_get_link(packet);
+	linkptr = trace_get_packet_buffer(packet,&linktype,NULL);
 	if (!linkptr) {
 		trace_set_err(packet->trace,
 				TRACE_ERR_BAD_PACKET,"Packet has no payload");
@@ -1005,7 +1080,6 @@ int trace_bpf_compile(libtrace_filter_t *filter,
 	
 	if (filter->filterstring && ! filter->flag) {
 		pcap_t *pcap = NULL;
-		libtrace_linktype_t linktype=trace_get_link_type(packet);
 		if (linktype==(libtrace_linktype_t)-1) {
 			trace_set_err(packet->trace,
 					TRACE_ERR_BAD_PACKET,
@@ -1036,7 +1110,7 @@ int trace_bpf_compile(libtrace_filter_t *filter,
 	}
 	return 0;
 #else
-	assert(!"This should never be called when BPF not enabled");
+	assert(!"Internal bug: This never be called when BPF not enabled");
 	trace_set_err(packet->trace,TRACE_ERR_OPTION_UNAVAIL,
 				"Feature unavailable");
 	return -1;
@@ -1045,9 +1119,9 @@ int trace_bpf_compile(libtrace_filter_t *filter,
 
 DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 			const libtrace_packet_t *packet) {
-#ifdef HAVE_BPF
+#ifdef HAVE_BPF_FILTER
 	void *linkptr = 0;
-	unsigned int clen = 0;
+	uint32_t clen = 0;
 	bool free_packet_needed = false;
 	int ret;
 	libtrace_packet_t *packet_copy = (libtrace_packet_t*)packet;
@@ -1075,7 +1149,7 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 		}
 	}
 	
-	linkptr = trace_get_link(packet_copy);
+	linkptr = trace_get_packet_buffer(packet_copy,NULL,&clen);
 	if (!linkptr) {
 		if (free_packet_needed) {
 			trace_destroy_packet(packet_copy);
@@ -1093,10 +1167,8 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 		return -1;
 	}
 
-	clen = trace_get_capture_length(packet_copy);
-
 	assert(filter->flag);
-	ret=bpf_filter(filter->filter.bf_insns,(u_char*)linkptr,clen,clen);
+	ret=bpf_filter(filter->filter.bf_insns,(u_char*)linkptr,(unsigned int)clen,(unsigned int)clen);
 	if (free_packet_needed) {
 		trace_destroy_packet(packet_copy);
 	}
@@ -1521,3 +1593,46 @@ void trace_construct_packet(libtrace_packet_t *packet,
 	memcpy(packet->payload,data,(size_t)len);
 	packet->type=pcap_linktype_to_rt(libtrace_to_pcap_linktype(linktype));
 }
+
+uint64_t trace_get_received_packets(libtrace_t *trace)
+{
+	assert(trace);
+	if (trace->format->get_received_packets) {
+		return trace->format->get_received_packets(trace);
+	}
+	return (uint64_t)-1;
+}
+
+uint64_t trace_get_filtered_packets(libtrace_t *trace)
+{
+	assert(trace);
+	if (trace->format->get_filtered_packets) {
+		return trace->format->get_filtered_packets(trace)+
+			trace->filtered_packets;
+	}
+	if (trace->format->get_received_packets
+		&& trace->format->get_dropped_packets) {
+		return 
+			((trace_get_received_packets(trace)
+			-trace_get_accepted_packets(trace))
+			-trace_get_dropped_packets(trace))
+			+trace->filtered_packets;
+	}
+	return trace->filtered_packets;
+}
+
+uint64_t trace_get_dropped_packets(libtrace_t *trace)
+{
+	assert(trace);
+	if (trace->format->get_dropped_packets) {
+		return trace->format->get_dropped_packets(trace);
+	}
+	return (uint64_t)-1;
+}
+
+uint64_t trace_get_accepted_packets(libtrace_t *trace)
+{
+	assert(trace);
+	return trace->accepted_packets;
+}
+
